-- -*- coding: utf-8 -*-
--
-- Adobe Photoshop Lightroom export plugin for Flickr
-- Copyright 2008 Jeffrey Friedl
--
-- http://regex.info/blog/photo-tech/lightroom-flickr/
--
local PluginVersion      = '20080714.47'
local PluginContactName  = 'Jeffrey Friedl'
local PluginContactEmail = 'jfriedl@yahoo.com'
local PluginUrl          = 'http://regex.info/blog/photo-tech/lightroom-flickr/'

--[[----------------------------------------------------------------------------

Version History:

20080714.47 -- Flick Pro users can now opt to <b>Replace</b> previously-uploaded images. I'm not 100% sure exactly
               what the semantics are with respect to what is preserved from the prior image and what is taken from the new
               image; let me know how it goes.

20080622.46 -- Fixed the "_update_status" bug that was still happening when the plugin was used
               with an export preset configured to save non-sRGB JPGs to a photoset
               selected at export time.

20080613.45 -- A couple of small bug fixes that might help this run in the 2.0 beta a bit better.

20080606.44 -- <p>Minor tweaks: Added the Flickr logo to the head of the export dialog.
               Moved the plugin debug log to the Documents / My Documents folder.</p>

               <p><b>Mac Users</b>: if you get an error when you try to install the plugin,
               something along the lines of "this plugin is already installed or not
               compatible with your version of Lightroom", please see the <a
               href='http://regex.info/blog/2007-11-17/638'>install instructions</a> and
               <b>manually remove</b> the previously-installed version(s). If that still
               doesn't let you install the new version by clicking, then manually install
               it. I have no idea why this is happening, so if someone figures it out,
               please let me know.</b>

20080413.43 -- Uh, a version of the .42 fix that actually fixed.

20080413.42 -- Fixed a problem that caused some people to get an internal error with the previous version.
               I also reverted the change added in .39 that was aimed to get around a problem in 1.4.0. That problem
               no longer exists in 1.4.1, so I've put things back.

20080411.41 -- Fix for the "_update_status" bug that those few who export non-sRGB JPGs ran into.

20080327.40 -- <p>The &#8220;Filesize was zero&#8221; bug has been solved. Sort of.</p>

               <p>It turns out that &#8220;Filesize was zero&#8221; is
               Flickr's shorthand for &#8220;The size of the file you
               attempted to upload
               ended up being larger than the per-image upload limit of your account,
               which is 5MB if your account is a free one, or 10MB for a pro
               account.&#8221;</p>

               <p>The plugin now checks the image that Lightroom builds for
               the export, and reports the error prior to uploading.
               The entire export is canceled at that point. I'll work on
               more sociable error handling for a future version.</p>

               <p>Thanks to Ken Rachynski for his help in debugging this.</p>

20080315.39 -- To work around a time-related bug in Lightroom 1.4, I've changed the option that used to be
               &#8220;<i>Set 'taken on' date with each shot when Minimize Embedded Metadata is set</i>&#8221; to one that
               always sets the proper time on the image. It's now &#8221;<b>Set 'taken on' date with each shot (to work around a Lightroom 1.4 bug)</b>&#8221;,
               so you'll probably want to set that if you're using Lightroom 1.4, unless you don't want any date-related metadata sent.

               Even with this option selected, the file sent to Flickr will still have the wrong &#8220;00:00:00&#&8221; time encoded in the EXIF,
               but the plugin will tell Flickr the proper time, so Flickr's idea of the image's time will be correct.


20080314.38 -- Minor update to allow the plugin to work with <a href='http://regex.info/blog/2008-03-14/762'>Lightroom 1.4</a>.

20080221.37 -- Minor UI tweaks for better appearance on a Mac.
               <br/>Updated Swedish translation.

20080217.36 -- Small fix to the wording in the Title/Description dialog

20080217.35 -- <ul><li>Updated French translation.</li>
               <li>Now includes a Swedish translation by <a href='http://nicke.sjostrom.fi/'>Niklas Sjöström</a>.</li>
               <li>The &#8220;select photoset at time of export&#8221; (for export presets) feature should actually work now.</li>
               </ul>

20080202.34 -- <p>Refactored a bunch of the photoset-related code &mdash; could have introduced some bugs.</p>

               <p>You can now sort the list of photosets, and for presets, have the photoset dialog come up with
               each export (so now you can select a photoset, or create a new photoset, from export presets).</p>

               <p>Includes New version (20080130.4) of my <a href='http://regex.info/blog/photo-tech/anycommand/'>Run Any Command</a> piglet.</p>

20080128.33 -- Tidied up <a
               href='http://regex.info/blog/photo-tech/anycommand/#comment-13764'>some
               mistakes in the translation hooks</a>.
               Interested in doing the translations for my plugins for your favorite language?
               It's a difficult and thankless job, but you earn good karma for giving to the greater community.
               If interested, please contact me (<b>jfriedl</b>&#8205;&#64;&#8205;<b>yahoo.com</b>).

20080123.31 -- Fix "internal error" bug introduced in previous build, that affected those upgrading from several versions  back.

20080122.30 -- Added <a href="http://regex.info/blog/photo-tech/anycommand/">"Run Any Command" Piglet</a>.
               <br/>Some internal refactoring to better support the <a href='http://regex.info/blog/2008-01-15/692'>piglet infrastructure</a>.

20080117.29 -- Have export dialog allow file types that any enabled piglets allow. Bail if the resulting filename
               has an extension that can't be uploaded.

20080115.28 -- Added the <a href='http://regex.info/blog/photo-tech/piglets/'>piglet manager</a> for fourth-party add-ons

20080109.27 -- New and tidied French translation

20080107.26 -- Housekeeping changes to allow auto-install on OSX

20080104.25 -- Add an option to have the "taken on" date sent with each
               image even when "Minimized Embedded Metadata" is checked in the Metadata
               section, to address the <a
               href='http://regex.info/blog/2007-11-17/638#comment-13163'>issue
               reported by Scott McMillin</a>

20071227.24 -- Updated French translation

20071222.23 -- Ensure that a long login status message does not crowd out the logout button

20071217.22 -- Update to the title/description logic.
               Properly handle a no-gallery situation.
               New French translation file.

20071213.21 -- fix bug introduced in .19

20071213.20 -- new French translation file

20071213.19 -- Can now filter the export so that only images previously exported to Flickr will be processed,
               or, conversely, only images that have not been previously exported to Flickr.

20071212.18 -- Minor internationalization prep.

20071210.17 -- More internationalization prep.  Added a first draft of a French translation, courtesy of
               <a href='http://www.utiliser-lightroom.fr'>Gilles Theophile</a>.
               (Contact him with any suggested changes.)

20071205.16 -- Can now export the original master file.
               Can now note an image resolution in pixels per inch/cm in the image metadata.

20071204.15 -- More internationalization prep.

20071204.14 -- Fixed keywording
               <br/>Added the ability to enter extra keywords in either Flickr style (space separated),
               or Lightroom style (comma separated).
               <br/>
               Lots of internal work to lay the groundwork for internationalizing the plugin.

20071127.12 -- Fix typo introduced in the previous version.
               <br/>Send along a per-install random identifier with the version check,
               so I can count how many of each plugin is being used.

20071127.11 -- Make it so that the previous item (bandwidth quota not shown for pros) works without
               requiring reauthentication.

20071127.10 -- Don't show upload quota remaining if the user has a pro account, since
               such accounts have unlimited bandwidth (or so I'm told... I don't have a pro account,
               so I really can't test this)

20071127.9 --  Fix typos introduced in previous version

20071126.8 --  <p>Added some HTTP error trapping in an effort to get around the 'FormatMessageW failed' error that
               some Windows users have been seeing:</p>

                <img class='screenshot' src='http://regex.info/i/lr/Trapping-Dialog.gif'/>

               <p>When the trapping is enabled, those errors are ignored, which may allow things to mostly work,
               or may simply lead to silent failure. Features that rely on
               the result of the upload (the new photoid) will stop
               working for uploads that experience a trapped error, such as
               the 'delete previously uploaded versions' feature.</p>

               <p>This may not help, but it's worth a try until the underlying issue can be found and resolved.</p>

20071125.7 --  Now handles properly ampersands in Flickr photoset names

20071125.6 --  Updated the Upload-Management section to allow visiting the exported photos only when
               not visiting the exported photoset, which allows one to always visit exactly one Flickr page
               after the upload, the photoset if a photoset was used, or the list of exported photos otherwise.

               <img class='screenshot' src='http://regex.info/i/lr/ExportPlugin-NewFlickrVisit.gif'/>

20071125.5 --  Added a "Save to Local Disk" section to the dialog:

               <img class='screenshot' src='http://regex.info/i/lr/ExportPlugin-SaveLocal.gif'/>

20071123.3 --  Added plugin debugging option, to track HTTP calls

20071123.2 --  Report under the Title/Description the result of what is chosen

20071121.1 --  Initial public release



--]]----------------------------------------------------------------------------


-- system imports
local LrPathUtils = import 'LrPathUtils'
local LrDialogs   = import 'LrDialogs'
local LrBinding   = import 'LrBinding'
local LrColor     = import 'LrColor'
local LrPrefs     = import 'LrPrefs'
local LrView      = import 'LrView'
local LrDate      = import 'LrDate'

uLOC = LOC -- this is needed for my automated build tools

-- local imports
local translator_info  = require 'translator_info'

local colorspace_check = require 'colorspace_check'(LOC("$$$/175=Note: Regardless, ^1 converts non-sRGB images to sRGB for display.", "Flickr"))

local caption_source   = require 'caption_source' { provider = 'Flickr',
                                                    sets = {
                                                       { tag = 'title',       label = uLOC("$$$/Flickr/Title=title"),             p1 = 'T', p2 = 'H', p3 = 'F', p4 = 'X' },
                                                       { tag = 'description', label = uLOC("$$$/Flickr/Description=description"), p1 = 'C', p2 = 'X', p3 = 'X', p4 = 'X' }
                                                    }
                                                 }
local version_check    = require 'version_check'('flickr', PluginVersion, PluginUrl)
local upload_filter    = require 'upload_filter'({ ID = 'photoIds', provider = 'Flickr' })
local save_to_disk     = require 'save_to_disk' { provider = 'Flickr' }
local resolution       = require 'resolution'
local async_task       = require 'async_task'
local tableutil        = require 'table-util'
local Piglets          = require 'piglet_manager.lua'
local util             = require 'util'
local API              = require 'API'
local extra_tags       = require 'extra_tags' {
                                                provider       = 'Flickr',
                                                default        = 'space',
                                                extra_tags_key = "flickr_extra_tags"
                                               }

-- this 'interactive_destination' variable must be global so that we can access it from API.lua
interactive_destination = require 'interactive_destination'({
                                                               provider      = 'Flickr',
                                                               text_target   = LOC("$$$/427=destination photoset"),
                                                               title_target  = LOC("$$$/428=Flickr Photoset"),
                                                               select_key    = 'flickr_destination_photoset'
                                                            })



local LrHttp           = API.LrHttp

local GRAY    = LrColor(.5,.5,.5)

--require 'jfriedlDebug.lua' -- defines Dump() and DebugNote()



--
-- Returns the definition for the Login part of the export dialog.
--
local function sectionsForTopOfDialog(v, state)

   state.LR_size_resolution = 200

   state._have_some_photosets = false
   state._new_set_name = ''
   state._new_set_desc = ''

   --
   -- state._buttons_are_enabled is a generic flag that disables all buttons when
   -- false. When true, it allows a button to become enabled if other
   -- button-specific conditions are met.
   --
   state._buttons_are_enabled = true

   --
   -- Initialize other temporary items we'll be accessing in 'state'
   --
   state._logging_in = false


   async_task.run(state, LOC("$$$/389=checking user data"), API.refreshUserDataWithoutAutoAuthentication)

   --
   -- A general "update the status of a bunch of stuff" routine.
   --
   local function update_status()

      state._fully_logged_out = not state._logging_in and state.flickr_auth_token == nil
      state._fully_logged_in  = not state._logging_in and state.flickr_auth_token ~= nil

      if not state.flickr_auth_token then

         state._account_synopsis = LOC("$$$/313=Not authenticated to ^1", "Flickr")
         state.LR_canExport = false
         state._account_status = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx' -- just reserve some space

      else

         if state.flickr_name then
            state._account_synopsis = LOC("$$$/314=Authenticated as ^1",                 tostring(state.flickr_name))
            state._account_status   = LOC("$$$/315=Authenticated to ^1 as ^2", "Flickr", tostring(state.flickr_name))
         else
            -- There are some transient periods where we're authenticated but don't have a name
            -- So that the user doesn't see "nil" flash by, we'll handle this special case
            state._account_synopsis = LOC("$$$/316=Authenticated")
            state._account_status   = LOC("$$$/317=Authenticated to ^1", "Flickr")
         end

         if state.flickr_is_pro then

            state._account_synopsis = state._account_synopsis .. LOC("$$$/318= (pro)")
            state._account_status   = state._account_status   .. LOC("$$$/318= (pro)")

         elseif state.flickr_bandwidth_remain then

            local meg = state.flickr_bandwidth_remain / 1024
            if meg >= 10 then
               meg = string.format(LOC("$$$/319=%.0f meg"), meg)
            elseif meg < 0.1 then
               meg = LOC("$$$/320=none")
            else
               meg = string.format(LOC("$$$/321=%.1f meg"), meg)
            end

            state._account_synopsis = state._account_synopsis .. ";   " .. LOC("$$$/322=quota remaining: ^1", meg)
            state._account_status   = state._account_status   .. "\n"   .. LOC("$$$/323=Monthly upload quota remaining: ^1", meg)

         end

         if state.flickr_destination_photoset == 'new' then
            if state._new_set_name == '' then
               state.LR_canExport = false
               state._destination_synopsis = LOC("$$$/324=AWAITING NEW SET NAME")
            else
               state.LR_canExport = Piglets.canExport(state)
               state._destination_synopsis = LOC("$$$/325=add to NEW ^1", state._new_set_name)
            end
         else
            state.LR_canExport = Piglets.canExport(state)
            if state.flickr_destination_photoset == '' then
               state._destination_synopsis = LOC("$$$/326=not added to a photoset")
            else
               if state.flickr_photoset2name[state.flickr_destination_photoset] then
                  state._destination_synopsis = LOC("$$$/327=add to ^1", state.flickr_photoset2name[state.flickr_destination_photoset])
               else
                  state._destination_synopsis = LOC("$$$/328=add to a photoset")
               end
            end
         end
      end

      local color_warning = colorspace_check.synopsis(state)
      if color_warning then
         if not state._destination_synopsis or state._destination_synopsis == '' then
            state._destination_synopsis = color_warning
         else
            state._destination_synopsis = color_warning .. "; " .. state._destination_synopsis
         end
      end
   end

   state._update_status = update_status -- throw a pointer here so that it's easily accessible from anywhere that 'state' is

   --
   -- Various reasons to recalculate the status...
   --
   state:addObserver('flickr_auth_token',           update_status)
   state:addObserver('flickr_name',                 update_status)
   state:addObserver('flickr_bandwidth_remain',     update_status)
   state:addObserver('_logging_in',                 update_status)
   state:addObserver('_new_set_name',               update_status)
   state:addObserver('flickr_destination_photoset', update_status)
   Piglets.set_CanExport_Observer(state, update_status)
   --
   -- Set the initial status
   --
   update_status()


   --
   -- Return the LrView hierarchy for the login stuff....
   --
   local Sections = {
      {
         title    = LOC("$$$/204=Your ^1 Account", "Flickr"),
         place    = 'vertical',
         synopsis = LrView.bind { key = '_account_synopsis', object = state },

         v:view {
            place = 'overlapping',
            fill_horizontal = 1,

            --
            -- the "while not logged in" view
            --
            v:view {
               place = 'horizontal',
               margin_bottom = util.WinMac(0,2),
               fill_horizontal = 1,
               visible = LrView.bind '_fully_logged_out',

               v:static_text { title = LOC("$$$/329=You are not authenticated to ^1", "Flickr") },

               v:push_button {
                  enabled = LrView.bind('_buttons_are_enabled'),
                  title = LOC("$$$/330=Authenticate to ^1", "Flickr"),
                  action = function(button)
                              state._logging_in = true
                              async_task.run(state, LOC("$$$/387=^1 login", 'Flickr'),
                                             -- function to run
                                             function (state)
                                                API.getAuthToken(state)
                                                API.refreshUserDataWithoutAutoAuthentication(state)
                                             end,

                                             -- extra arg
                                             nil, 

                                             -- cleanup function
                                             function(state)
                                                state._logging_in = false
                                             end)
                           end,
               },

               v:static_text { title = "  " }, -- this should be a spacer

               v:push_button {
                  title = LOC("$$$/209=Visit ^1", "Flickr"),
                  action = function(button)
                              LrHttp.openUrlInBrowser("http://flickr.com/")
                           end,
                  tooltip = LOC("$$$/110=Press to open ^1 in your browser", "http://flickr.com/"),
               },
            },


            --
            -- the "while logging in..." view...
            --
            v:view {
               visible = LrView.bind('_logging_in'),
               place = 'vertical',
               fill_horizontal = 1,

               v:static_text {
                  title = LOC("$$$/210=Logging in to ^1^.", "Flickr"),
               },
               v:static_text {
                  title = LrView.bind('_account_synopsis'),
               },
            },


            --
            -- the "while logged in" view
            --
            v:view {
               visible = LrView.bind '_fully_logged_in',
               place = 'horizontal',
               margin = util.WinMac(0, 2),
               spacing = v:label_spacing(),
               fill_horizontal = 1,

               v:static_text {
                  title = LrView.bind('_account_status'),
                  height_in_lines = 2,
                  --width_in_chars = util.WinMac(48,39),
               },

               v:push_button {
                  visible = LrBinding.keyIsNotNil('flickr_home_url'),
                  title = LOC("$$$/211=Visit Your ^1", "Flickr"),
                  action = function(button)
                              LrHttp.openUrlInBrowser(state.flickr_home_url)
                           end,
               },


               v:push_button {
                  enabled = LrView.bind('_buttons_are_enabled'),
                  title = LOC("$$$/212=Logout"),
                  action = function(button)
                              state._buttons_are_enabled = false
                              API.logout(state)
                              state._buttons_are_enabled = true
                           end,
               },
            },
         },
      },
   }

   Piglets.appendTopSections(Sections, state, v)

   return Sections
end



local function creategallery_dialog(v, state, context, selection_item)

   if context == 'interactive destination' then
      state._buttons_are_enabled = true
      assert(selection_item)

      local STATE = LrPrefs.prefsForPlugin(_PLUGIN.id)
      state:addObserver(selection_item, function()
                                           state.flickr_destination_photoset_at_export = state[selection_item]
                                           STATE.flickr_destination_photoset_at_export = state[selection_item]
                                       end)
   end

   local function get_items_to_use()

      if context ~= 'interactive destination' then
         selection_item = 'flickr_destination_photoset'

         state._items_to_use   = state.flickr_photosets
      else
         state._items_to_use = interactive_destination.filter_items(state.flickr_photosets)
            
         if state.flickr_destination_photoset_at_export ~= "" and util.popupmenu_items_has_value(state._items_to_use, state.flickr_destination_photoset_at_export) then
            state[selection_item] = state.flickr_destination_photoset_at_export
         elseif state.flickr_destination_photoset ~= "" and util.popupmenu_items_has_value(state._items_to_use, state.flickr_destination_photoset) then
            state[selection_item] = state.flickr_destination_photoset
         else
            state[selection_item] = state._items_to_use[1].value
         end
      end

   end


   get_items_to_use()
   state:addObserver('flickr_photosets',            get_items_to_use)
   state:addObserver('flickr_destination_photoset', get_items_to_use)
   

   state:addObserver('flickr_photoset_sort',  function()
                                                 API.sort_photoset_list(state)
                                              end)
--   --
--   -- Enable the create-gallery button when the user starts typing a gallery name.
--   --
--   local function check_gallery_name()
--      state._enable_create_gallery_button = state._new_gallery_name ~= ''
--   end
--   state:addObserver('_new_gallery_name', check_gallery_name)
--   check_gallery_name()


   local view_guts = {
      bind_to_object = state,

      v:view {
         place = 'overlapping',

         v:static_text {
            visible = LrView.bind '_fully_logged_out',
            title = LOC("$$$/313=Not authenticated to ^1", "Flickr"),
         },

         v:view {
            visible = LrView.bind '_fully_logged_in',
            spacing = util.WinMac(0,2),
            place = 'vertical',

            v:view {
               place = 'horizontal',
               margin_bottom = util.WinMac(0,1),
               v:static_text {
                  title = LOC("$$$/343=Add images to your ^1 photoset: ", "Flickr"),
                  width = LrView.share "_x2",
               },

               v:view {
                  place = 'overlapping',
                  margin_bottom = 3,

                  v:static_text {
                     visible = LrBinding.keyIsNotNil '_getting_photoset_list',
                     title = LOC("$$$/426=working^."),
                     text_color = LrColor(1, .2, .1),
                  },
                  
                  v:popup_menu {
                     visible = LrBinding.keyIsNil '_getting_photoset_list',
                     items = LrView.bind('_items_to_use'),
                     value = LrView.bind(selection_item),
                     width_in_chars = 30,
                     tooltip = LOC("$$$/221=This list is refreshed each time you open the Export dialog"),
                  },
               },
            },

            v:view {
               spacing = 0,
               margin = 0,
               margin_bottom = 5,
               visible = LrView.bind('flickr_sortable_photosets'),

               place='horizontal',
               v:radio_button {
                  size = 'mini',
                  title = LOC("$$$/295=sort alphabetically"),
                  value = LrView.bind('flickr_photoset_sort'),
                  checked_value = 'Title',
               },
               v:spacer {
                  width = 15,
               },
               v:radio_button {
                  size = 'mini',
                  title = LOC("$$$/296=most recent first"),
                  value = LrView.bind('flickr_photoset_sort'),
                  checked_value = 'Time',
               },
            },

            v:view {
               place = 'overlapping',

               v:static_text { title = LOC("$$$/344=(You may not create additional photosets because your Flickr photoset quota has been used)"),
                               text_color = LrColor(.8,0,0),
                               visible = LrBinding.negativeOfKey('flickr_can_create_new_sets'), },

               v:view {
                  place = 'vertical',
                  visible = LrBinding.keyEquals(selection_item, 'new'),
                  margin_top = util.WinMac(0,4),
                  spacing    = util.WinMac(0,4),

                  v:view {
                     place = 'horizontal',
                     v:static_text { title = LOC("$$$/345=New set name: "),
                                     alignment = 'right',
                                     width = LrView.share "_x2" },

                     v:edit_field { value = LrView.bind '_new_set_name',
                                    width_in_chars = 30 },
                  },

                  v:view {
                     place = 'horizontal',
                     v:static_text { title = LOC("$$$/346=Optional description: "),
                                     alignment = 'right',
                                     width = LrView.share "_x2", },

                     v:edit_field { value = LrView.bind '_new_set_desc',
                                    height_in_lines = 2,
                                    width_in_chars = 30},
                  }
               }
            }
         }
      }
   }

   if context ~= 'interactive destination' then

      table.insert(view_guts, colorspace_check.dialog_section(v, state))

   else

      --
      -- this is ugly
      --
      local function update_status()
         state._fully_logged_out = not state._logging_in and state.flickr_auth_token == nil
         state._fully_logged_in  = not state._logging_in and state.flickr_auth_token ~= nil
      end
      update_status()
      state:addObserver('_logging_in', update_status)
      state:addObserver('flickr_auth_token', update_status)

   end

   return v:view(view_guts)
end


interactive_destination.args().creategallery_dialog = creategallery_dialog
interactive_destination.args().refresh_function = API.getPhotosets

--
-- Returns the definition for the non-Login Flickr-specific parts of the export dialog.
--
local function sectionsForBottomOfDialog(v, state)

   -- Handle checking for caption-source changes (this must be called before the management_synopsis stuff)
   caption_source.watch(state)

   local function set_metadata_synopsis()

      local notes = {}

      local tag_note = extra_tags.check(state)
      if tag_note then
         table.insert(notes, tag_note)
      end
      
      table.insert(notes, state.flickr_viewable_by)

      if state.flickr_safety_level == '2' then table.insert(notes, LOC("$$$/331=Moderate")) end
      if state.flickr_safety_level == '3' then table.insert(notes, LOC("$$$/332=Restricted")) end
      if state.flickr_content_type == '2' then table.insert(notes, LOC("$$$/333=Screenshots")) end
      if state.flickr_content_type == '3' then table.insert(notes, LOC("$$$/334=Art/other")) end
      if state.flickr_hide_from_public    then table.insert(notes, LOC("$$$/335=Unsearchable")) end

      state._other_metadata_synopsis = table.concat(notes, ';   ')
   end

   state:addObserver('flickr_content_type',     set_metadata_synopsis)
   state:addObserver('flickr_hide_from_public', set_metadata_synopsis)
   state:addObserver('flickr_safety_level',     set_metadata_synopsis)
   state:addObserver('flickr_viewable_by',      set_metadata_synopsis)
   set_metadata_synopsis()

   --
   -- Stuff that handles the synopsis of the "Upload management" section
   --
   local function set_management_synopsis()
      local notes = {}

      if state._upload_filter_synopsis then
         table.insert(notes, state._upload_filter_synopsis)
      end

      if state._upload_filter_can_do_old then

         if state.flickr_delete_prior_uploads then
            table.insert(notes, LOC("$$$/336=previous are DELETED"))
         else
            table.insert(notes, LOC("$$$/337=previous are retained"))
         end
      end

      if state.flickr_visit_photos_after_export and state.flickr_visit_photoset_after_export then
         if state.flickr_visit_photos_after_export_mutex then
            table.insert(notes, LOC("$$$/338=visit photos or photoset"))
         else
            table.insert(notes, LOC("$$$/339=visit photos and photoset"))
         end
      elseif state.flickr_visit_photos_after_export then
         table.insert(notes, LOC("$$$/340=visit photos"))
      elseif state.flickr_visit_photoset_after_export then
         table.insert(notes, LOC("$$$/341=visit photoset"))
      end
      
      state._upload_management_synopsis = table.concat(notes, ';   ')
   end
   
   state:addObserver('flickr_visit_photos_after_export',   set_management_synopsis)
   state:addObserver('flickr_visit_photoset_after_export', set_management_synopsis)
   state:addObserver('flickr_delete_prior_uploads',        set_management_synopsis)
   state:addObserver('_upload_filter_synopsis',            set_management_synopsis)
   
   -- set the initial synopsis
   set_management_synopsis()

   local replace_enabled
   local replace_value
   local replace_text = LOC("$$$/607=Replace the most-recently previously-uploaded version of the exported images in your ^1 account", "Flickr")
   local replace_note

   if state.flickr_is_pro then
      replace_enabled = LrView.bind('_upload_filter_can_do_old')
      replace_value = LrView.bind('flickr_replace_prior_uploads')
      replace_note = "dummy"
   else
      replace_enabled = false
      replace_value = false
      replace_note = v:view {
         margin_left = 20,
         v:static_text {
            title = LOC("$$$/608=(available only to Flickr Pro accounts)"),
            enabled = false
         }
      }
   end

   local Sections = {
      --
      -- ADD TO PHOTOSET section
      --
      {
         title = "Flickr: " .. LOC("$$$/342=Add to Photoset"),
         synopsis = LrView.bind { key = '_destination_synopsis', object = state },
         spacing = 0,

         creategallery_dialog(v, state, 'export dialog')
      },
      
      save_to_disk.dialog_section(v, state),

      --
      -- TITLE and DESCRIPTION section
      --
      {
         title = "Flickr: " .. LOC("$$$/347=Title / Description"),
         
         v:column {
            v:static_text { title = LOC("$$$/239=Title:"), font = "<system/bold>" },
            caption_source.dialog_section(v, state, 'title'),
         },
         v:column {
            v:static_text { title = LOC("$$$/240=Description:"), font = "<system/bold>" },
            caption_source.dialog_section(v, state, 'description'),
         },
      },


      --
      -- OTHER METADATA
      --
      {
         title = "Flickr: " .. LOC("$$$/348=Other Metadata"),
         synopsis = LrView.bind { key = '_other_metadata_synopsis', object = state },

         extra_tags.dialog_section(v, state, set_metadata_synopsis),

         v:row {
            v:static_text { title = LOC("$$$/349=Allow to be viewed by: ") },

            v:radio_button {
               title = LOC("$$$/350=Public" .."  "),
               checked_value = "Public",
               value = LrView.bind("flickr_viewable_by")
            },
            v:radio_button {
               title = LOC("$$$/351=Friend/Family").. "  ",
               checked_value = "Friends & Family",
               value = LrView.bind("flickr_viewable_by")
            },
            v:radio_button {
               title = LOC("$$$/352=Friends") .."  ",
               checked_value = "Friends",
               value = LrView.bind("flickr_viewable_by")
            },
            v:radio_button {
               title = LOC("$$$/353=Family") .. "  ",
               checked_value = "Family",
               value = LrView.bind("flickr_viewable_by")
            },
            v:radio_button {
               title = LOC("$$$/354=No one else"),
               checked_value = "X",
               value = LrView.bind("flickr_viewable_by")
            },
         },


         v:row {
            v:static_text { title = LOC("$$$/355=The ^[safety level^] of these images: ") },

            v:radio_button {
               title = LOC("$$$/356=Safe") .."  ",
               checked_value = "1",
               value = LrView.bind("flickr_safety_level")
            },
            v:radio_button {
               title = LOC("$$$/331=Moderate") .. "  ",
               checked_value = "2",
               value = LrView.bind("flickr_safety_level")
            },
            v:radio_button {
               title = LOC("$$$/332=Restricted"),
               checked_value = "3",
               value = LrView.bind("flickr_safety_level")
            }
         },

         v:row {
            v:static_text { title = LOC("$$$/357=Type of images: ") },

            v:radio_button {
               title = LOC("$$$/358=Photos") .. "  ",
               checked_value = "1",
               value = LrView.bind("flickr_content_type")
            },
            v:radio_button {
               title = LOC("$$$/333=Screenshots") .. "  ",
               checked_value = "2",
               value = LrView.bind("flickr_content_type")
            },
            v:radio_button {
               title = LOC("$$$/359=Other (art/CGI/illustration/etc.)"),
               checked_value = "3",
               value = LrView.bind("flickr_content_type")
            }
         },

         v:checkbox {
            title = LOC("$$$/360=Hide these images from Flickr's public search"),
            value = LrView.bind('flickr_hide_from_public'),
         },

         resolution.dialog_section(v, state),

         v:view {
            place = 'horizontal',
            spacing = 2,

            v:checkbox {
               title   = "",
               enabled = LrView.bind('LR_minimizeEmbeddedMetadata'),
               value   = LrView.bind('flickr_send_photo_date'),
            },
            v:static_text {
               enabled = LrView.bind('LR_minimizeEmbeddedMetadata'),
               title = LOC("$$$/423=Set ^[taken on^] date even when ^[Minimize Embedded Metadata^] is checked in the Metadata section")
               --title = LOC("$$$/423=Set ^[taken on^] date with each shot (to work around a Lightroom 1.4 bug)")
            },
         },
      },

      --
      -- UPLOAD MANAGEMENT section
      --
      {
         title = "Flickr: " .. LOC("$$$/241=Upload Management"),
         synopsis = LrView.bind { key = '_upload_management_synopsis', object = state },

         upload_filter.dialog_section(v, state),

         v:spacer { height = 8 },

         spacing = 0,

         v:checkbox {
            enabled = replace_enabled,
            title   = replace_text,
            value   = replace_value,
         },

         replace_note,

         v:spacer { height = 8 },

         v:checkbox {
            enabled = LrView.bind('_upload_filter_can_do_old'),
            title   = LOC("$$$/242=Delete any previously-uploaded versions of the exported images in your ^1 account", "Flickr"),
            value   = LrView.bind('flickr_delete_prior_uploads'),
            tooltip = LOC("$$$/243=If this option is checked, after uploading an image, any versions of the same image previously uploaded to ^1 (via this plugin) are deleted from ^1.", "Flickr"),
         },

         v:spacer { height = 8 },

         v:checkbox {
            title   = LOC("$$$/361=Visit target photoset after upload"),
            value   = LrView.bind('flickr_visit_photoset_after_export'),
            tooltip = LOC("$$$/362=If this option is checked, the target photoset (if any) is visited at Flickr after the export has completed."),
         },

         v:spacer { height = 8 },

         v:view {
            place = 'horizontal',
            spacing = 0,

            v:checkbox {
               title   = LOC("$$$/363=Visit uploaded photos after upload^."),
               value   = LrView.bind('flickr_visit_photos_after_export'),
               tooltip = LOC("$$$/364=If this option is checked, the uploaded images are visited at Flickr after the export has completed."),
            },

            v:static_text {
               enabled = LrBinding.andAllKeys('flickr_visit_photoset_after_export', 'flickr_visit_photos_after_export'),
               title = LOC("$$$/365=but only when not visiting a photoset page")
            },

            v:checkbox {
               enabled = LrBinding.andAllKeys('flickr_visit_photoset_after_export', 'flickr_visit_photos_after_export'),
               title = "",
               value = LrView.bind('flickr_visit_photos_after_export_mutex'),
            }
         }
      }
   }

   --
   -- PLUGIN INFO section
   --
   local PluginInfoSection = {
      title = LOC("$$$/249=^1 Export Plugin Info", "Flickr"),
      synopsis = LrView.bind { key = 'info_synopsis', object = state },
      spacing = 0,
      fill_horizontal = 1,

      v:view {
         place = 'horizontal',
         margin_bottom = 0,

         v:static_text {
            title = LOC("$$$/250=^1 Export Plugin for Lightroom", "Flickr"),
            font = "<system/bold>",
         },

         v:static_text { title = LOC("$$$/251=version ^1", PluginVersion) },
      },

      v:view {
         margin_left   = 10,
         spacing       = util.WinMac(0,4),
         margin_bottom = util.WinMac(0,2),

         v:static_text {
            title = " " .. LOC("$$$/366=by ^1", PluginContactName),
            tooltip = PluginContactEmail,
         },

         v:push_button {
            size = "small",
            title = "  " .. PluginUrl .. "  ",
            action = function(button)
                        LrHttp.openUrlInBrowser(PluginUrl)
                     end,
         },
      },

      translator_info.dialog_section(v, state),

      v:view {
         margin_top = 10,

         v:static_text {
            -- This must be translated as per section 3(b) of the various language versions of http://flickr.com/services/api/tos/
            title = LOC("$$$/367=This product uses the Flickr API but is not endorsed or certified by Flickr"),
            text_color = GRAY,
         },
      },

      version_check.dialog_section(v, state),

      v:view {
         fill_horizontal = 1,
         place = 'horizontal',
         spacing = 0,
         margin_top = 10,
         margin_bottom = 10,
         Piglets.button(v, state),

         v:spacer { fill_horizontal = 1 },
         v:static_text {
            title = LOC("$$$/x1253=Donations are not required, but may be made"),
            text_color = LrColor(.6)
         },
         v:push_button {
            size = 'mini',
            title = LOC("$$$/x1254=here"),
            action = function(button)
                        LrHttp.openUrlInBrowser("https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=jfriedl%40yahoo%2ecom&item_name=Jeffrey%20Friedl%27s%20Lightroom%20Plugin%20Development&item_number=Flickr+Plugin&no_shipping=1&cn=Comment&tax=0&currency_code=USD&lc=US&bn=PP%2dDonationsBF&charset=UTF%2d8")
                     end
         }
      },

      LrHttp.dialog_section(state, v),
   }

   Piglets.appendBottomSections(Sections, state, v)
   table.insert(Sections, PluginInfoSection)

   return Sections
end


local allowed_file_extensions = {
   jpg = 1, jpeg = 1,
}

local function do_one_upload(PIGLET_INFO, args)

   local simpleImageName = LrPathUtils.leafName(PIGLET_INFO.image_path)
   local state = PIGLET_INFO.state

   if not PIGLET_INFO.success then
      local ErrorMessage = PIGLET_INFO.image_path
      LrDialogs.message(LOC("$$$/265=Couldn't render ^1", simpleImageName), ErrorMessage, "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'
   end

   if not allowed_file_extensions[string.lower(LrPathUtils.extension(PIGLET_INFO.image_path))] then
      local extension = LrPathUtils.extension(PIGLET_INFO.image_path)
      LrDialogs.message(LOC("$$$/424=^1 does not accept ^[.^2^] files for upload; aborting.", "Flickr", extension),
                        PIGLET_INFO.image_path,
                        "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'
   end         

   local ARGS = {}
   ARGS.safety_level = state.flickr_safety_level
   ARGS.content_type = state.flickr_content_type
   if state.flickr_hide_from_public then
      ARGS.hidden = '2'
   end

   if state.flickr_viewable_by:match('Public') then
      ARGS.is_public = '1'
   else
      ARGS.is_public = '0'

      if state.flickr_viewable_by:match('Friends') then
         ARGS.is_friend = '1'
      else
         ARGS.is_friend = '0'
      end

      if state.flickr_viewable_by:match('Family') then
         ARGS.is_family = '1'
      else
         ARGS.is_family = '0'
      end

   end

   local keyword_string
   local photoTimeStamp
   ARGS.photo_id = nil

   PIGLET_INFO.exportContext.exportSession.catalog:withCatalogDo(function()

                                                        local caption = caption_source.get(state, PIGLET_INFO.rendition.photo, simpleImageName, 'title')
                                                        if caption and caption ~= '' then
                                                           ARGS.title = caption
                                                        end

                                                        local desc = caption_source.get(state, PIGLET_INFO.rendition.photo, simpleImageName, 'description')
                                                        if desc and desc ~= '' then
                                                           ARGS.description = desc
                                                        end

                                                        keyword_string = PIGLET_INFO.rendition.photo:getFormattedMetadata('keywordTags')

                                                        photoTimeStamp = PIGLET_INFO.rendition.photo:getRawMetadata('dateTime')

                                                        if state.flickr_is_pro and state.flickr_replace_prior_uploads then
                                                           PIGLET_INFO.rendition.photo:withSettingsForPluginDo(_PLUGIN.id,
                                                                                                               function (imageState)
                                                                                                                  if imageState.photoIds and #imageState.photoIds > 0 then
                                                                                                                     ARGS.photo_id = imageState.photoIds[#imageState.photoIds]
                                                                                                                  end
                                                                                                               end)
                                                        end
                                                     end)

   --
   -- Calculate tags.
   --
   do
      local tags = {}

      if keyword_string ~= '' then
         tags = util.CsvToTable_NoEmpty(keyword_string)
      end

      if state.flickr_extra_tags then
         local extra = extra_tags.as_table(state)
         tags = tableutil.union(tags, extra)
      end

      ARGS.tags = table.concat(extra_tags.quote_all(tags), ' ')
   end


   local XmlReply = API.uploadPhoto(state, PIGLET_INFO.image_path, ARGS)

   if XmlReply == "TRAPPED ERROR" then

      -- oh well, let's pretend it worked and hope for the best

   elseif API.errorCode then
      if API.errorCode == 9998 then
         -- already reported in API.uploadPhoto
      elseif API.errorMessage == "Photo not found" then
         LrDialogs.message(LOC("$$$/600=Couldn't replace photo ^[^1^]", simpleImageName),
                           LOC("$$$/601=Original upload was likely deleted at Flickr prior to this replace attempt"),
                           "warning")
      else
         LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), LOC("$$$/368=Bad upload: ^1", API.errorMessage), "warning")
      end
      PIGLET_INFO.progressScope:cancel()
      return 'error'

   else

      local photoId = XmlReply:match('<photoid[^>]*>([0-9]+)</photoid>')

      if not photoId then
         LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), LOC("$$$/369=Can't grok XML:") .. "\n" .. XmlReply, "warning")
         PIGLET_INFO.progressScope:cancel()
         return 'error'
      end

      if PIGLET_INFO.DEST_PHOTOSET == 'new' then

         local photosetId = API.createPhotoset(state, state._new_set_name, state._new_set_desc, photoId)
         local registerdState = LrPrefs.prefsForPlugin(_PLUGIN.id)

         if photosetId then

            -- Changes we make in processRenderedPhotos to exportContext.propertyTable (aka "state")
            -- do not necessarily get reflected back to the LR catalog, so we have to be sure to
            -- save them via LrPrefs as well

            -- we update flickr_photoset2name[photosetId] first, which (as per LrPrefs docs) might not
            -- actually get noticed. Then, we update a root-level item (state.flickr_destination_photoset),
            -- which is sure to trigger a full copy.
            state.flickr_photoset2name[photosetId] = state._new_set_name
            registerdState.flickr_photoset2name[photosetId] = state._new_set_name

            PIGLET_INFO.DEST_PHOTOSET = photosetId
            if state.flickr_destination_photoset == 'new' then
               -- this was a photoset created from the main dialog, and not a "choose at export time" dialog
               state.flickr_destination_photoset          = photosetId
               registerdState.flickr_destination_photoset = photosetId
            end

         else

            PIGLET_INFO.DEST_PHOTOSET = ''
            state.flickr_destination_photoset = ''
            registeredState.flickr_destination_photoset = ''

            local UserReply = LrDialogs.confirm(LOC("$$$/370=Couldn't create ^[^1^] photoset", state._new_set_name),
                                                API.errorMessage,
                                                LOC("$$$/130=Continue With Export"),
                                                LOC("$$$/371=Abort Export"))
            if UserReply == 'cancel' then
               API.deletePhoto(state, photoId)
               PIGLET_INFO.progressScope:cancel()
               return 'error'
            end
         end
      end

      if PIGLET_INFO.DEST_PHOTOSET ~= '' then
         API.addToPhotoset(state, photoId, PIGLET_INFO.DEST_PHOTOSET)
      end

      PIGLET_INFO.exportContext.exportSession.catalog:withCatalogDo(function()

                                                             PIGLET_INFO.rendition.photo:withSettingsForPluginDo(_PLUGIN.id,

                                                                                                   function (imageState)

                                                                                                      if not imageState.photoIds then
                                                                                                         imageState.photoIds = {}
                                                                                                      end

                                                                                                      if state.flickr_delete_prior_uploads then
                                                                                                         for i, Id in ipairs(imageState.photoIds) do
                                                                                                            if not ARGS.photo_id or Id ~= ARGS.photo_id then
                                                                                                               API.deletePhoto(state, Id)
                                                                                                            end
                                                                                                         end
                                                                                                         imageState.photoIds = {}
                                                                                                      end

                                                                                                      table.insert(imageState.photoIds, photoId)
                                                                                                   end)
                                                        end)


      --
      -- If the user minimized embedded metadata, Flickr never gets the photo date, so we'll be sure to send
      -- at least that much along.
      --

      if state.LR_minimizeEmbeddedMetadata and state.flickr_send_photo_date then
      --if state.flickr_send_photo_date then
         -- Flickr date format is like "2004-11-29 16:01:26" (http://flickr.com/services/api/misc.dates.html)
         API.setPhotoDate(state, photoId, LrDate.timeToUserFormat(photoTimeStamp, '%Y-%m-%d %H:%M:%S'))
      end

      args.visitUrl = args.visitUrl .. photoId .. ','

   end

   Piglets.endOfOneRender(PIGLET_INFO)

   --
   -- We've uploaded the local file, so now we can move it somewhere if
   -- we want, delete it if we want, or just leave it be and it'll be
   -- deleted by LR automatically once the export is done.
   --
   save_to_disk.handleFile(state, PIGLET_INFO.image_path, simpleImageName)

   return 'okay'
end

local function processRenderedPhotos(functionContext, exportContext)

   if not upload_filter.apply(exportContext) then
      return
   end

   LrHttp.trapped_count = 0

   local state = exportContext.propertyTable
   local total = exportContext.exportSession:countRenditions()

   if not state.flickr_auth_token then
      LrDialogs.message(LOC("$$$/313=Not authenticated to ^1", "Flickr"),
                        LOC("$$$/305=Aborting export"),
                        "critical")
      return
   end

   local PIGLET_INFO = {
      state         = state,
      exportContext = exportContext,
      DEST_PHOTOSET = interactive_destination.get(state, total),
   }

   if not PIGLET_INFO.DEST_PHOTOSET then
      return
   end

   if not Piglets.priorToRender(PIGLET_INFO) then
      return
   end

   if not save_to_disk.preRenderStuff(state) then
      return
   end

   PIGLET_INFO.progressScope = exportContext:configureProgress{
      title = total == 1
         and  LOC("$$$/263=Uploading one image to ^1", "Flickr")
         or   LOC("$$$/264=Uploading ^1 images to ^2", tostring(total), "Flickr")
   }

   local success_count = 0
   local upload_count = 0

   local args = {
      visitUrl       = 'http://www.flickr.com/tools/uploader_edit.gne?ids='
   }

   for i, rendition in exportContext:renditions { stopIfCanceled = true } do

      PIGLET_INFO.rendition = rendition
      PIGLET_INFO.success, PIGLET_INFO.image_path = rendition:waitForRender()

      local continue = Piglets.startOfOneRender(PIGLET_INFO)
      if not continue then
         break
      end

      if PIGLET_INFO.progressScope:isCanceled() then
         break
      end

      if continue == 'skip this image' then

         -- it's not an error, so consider it a success
         success_count = success_count + 1

      else

         local status = do_one_upload(PIGLET_INFO, args)
         if status == 'okay' then
            upload_count  = upload_count  + 1
            success_count = success_count + 1
         end

      end

      if PIGLET_INFO.progressScope:isCanceled() then
         break
      end

   end

   local errors = total - success_count

   PIGLET_INFO.image_path     = nil
   PIGLET_INFO.success        = nil
   PIGLET_INFO.rendition      = nil
   PIGLET_INFO.attempted      = total
   PIGLET_INFO.exportedOkay   = success_count
   PIGLET_INFO.exportedError  = errors

   if PIGLET_INFO.progressScope:isCanceled() then
      Piglets.exportWasCanceled(PIGLET_INFO)
      return
   end


   if errors == 1 then
      LrDialogs.message(LOC("$$$/273=Upload Failure"), LOC("$$$/274=One image failed to upload"), "warning")
   elseif errors > 1 then
      LrDialogs.message(LOC("$$$/273=Upload Failure"), LOC("$$$/275=^1 images failed to upload", tostring(errors)), "warning")
   end

   LrHttp.IssueWarning()

   save_to_disk.postRenderStuff(state)

   if success_count > 0  then
      -- only show web pages if there was some success
      local visited_photoset = false
      if state.flickr_visit_photoset_after_export and PIGLET_INFO.DEST_PHOTOSET ~= '' then
         LrHttp.openUrlInBrowser(state.flickr_home_url .. "/sets/" .. PIGLET_INFO.DEST_PHOTOSET)
         visited_photoset = true
      end
      if state.flickr_visit_photos_after_export and (not state.flickr_visit_photos_after_export_mutex or not visited_photoset) then
         LrHttp.openUrlInBrowser(args.visitUrl)
      end
   end

   Piglets.exportComplete(PIGLET_INFO)
end

--
-- Finally, we return this to Lightroom to inform it about our export service.
--
return {
   --
   -- Flickr-specific configuration of the standard Export dialog sections...
   --
   allowFileFormats = Piglets.allowFileFormats { 'JPEG', 'ORIGINAL' },

   hideSections = {
      'exportLocation',
      'postProcessing',
   },

   hidePrintResolution = true,

   --
   -- Branding for the export-destination part of the export dialog
   --
   medium_icon = 'flickr_logo.png',
   title_alignment = 'right',
   title_color = LrColor(0/255, 99/255, 220/255), -- Flickr blue

   --
   -- init
   --
   startDialog = function(state)
                    Piglets.startDialog(state)
                    upload_filter.init(state)
                 end,

   endDialog   = function(state)
                    Piglets.endDialog(state)
                 end,


   --
   -- Flickr-specific parts of the dialog
   --
   sectionsForTopOfDialog    = sectionsForTopOfDialog,
   sectionsForBottomOfDialog = sectionsForBottomOfDialog,

   --
   -- "Upload to Flickr" post-render handler
   --
   processRenderedPhotos = processRenderedPhotos,

   --
   -- Data that I want to keep around from invocation to invocation
   --
   exportPresetFields = tableutil.union(
      {
         { key = 'flickr_auth_token',                      default = nil                                },
         { key = 'flickr_name',                            default = nil                                },
         { key = 'flickr_home_url',                        default = nil                                },
         { key = 'flickr_bandwidth_max',                   default = nil                                },
         { key = 'flickr_bandwidth_used',                  default = nil                                },
         { key = 'flickr_bandwidth_remain',                default = nil                                },
         { key = 'flickr_filesize_max',                    default = nil                                },
         { key = 'flickr_can_create_new_sets',             default = nil                                },
         { key = 'flickr_is_pro',                          default = nil                                },
         { key = 'flickr_safety_level',                    default = '1'                                },
         { key = 'flickr_content_type',                    default = '1'                                },
         { key = 'flickr_hide_from_public',                default = false                              },
         { key = 'flickr_viewable_by',                     default = 'Public'                           },
         { key = 'flickr_extra_tags',                      default = ''                                 },
         { key = 'flickr_visit_photos_after_export',       default = true                               }, -- if true, visit list of photos after export
         { key = 'flickr_visit_photoset_after_export',     default = true                               }, -- if true, visit target photoset after export
         { key = 'flickr_visit_photos_after_export_mutex', default = true                               }, -- show photos only if not showing a photoset
         { key = 'flickr_delete_prior_uploads',            default = false                              }, -- delete any previously-uploaded versions of an exported photo
         { key = 'flickr_replace_prior_uploads',           default = false                              },
         { key = 'flickr_photosets',                       default = API.default_photoset_list()        },
         { key = 'flickr_destination_photoset',            default = ''                                 },
         { key = 'flickr_destination_photoset_at_export',  default = ''                                 },
         { key = 'flickr_sortable_photosets',              default = false,                             },
         { key = 'flickr_photoset2name',                   default = {}                                 },
         { key = 'flickr_send_photo_date',                 default = true                               },
         { key = 'flickr_photoset_sort',                   default = 'Title'                            },
         { key = 'plugin_debugging',                       default = false                              },
         { key = 'first_version',                          default = PluginVersion                      },

      },
      interactive_destination.exportPresetFields(),
      colorspace_check.exportPresetFields(),
      caption_source.exportPresetFields(),
      version_check.exportPresetFields(),
      upload_filter.exportPresetFields(),
      save_to_disk.exportPresetFields(),
      resolution.exportPresetFields(),
      extra_tags.exportPresetFields(),
      Piglets.exportPresetFields(),
      LrHttp.exportPresetFields()
   ),
}
