--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
-- 

-- system imports
local LrDialogs = import 'LrDialogs'
local LrBinding = import 'LrBinding'
local LrColor   = import 'LrColor'
local LrView    = import 'LrView'
local LrHttp    = import 'LrHttp'

-- local plugin imports
local util      = require 'util.lua'


local colorspace_check = {}

function colorspace_check.show_warning_dialog(state)

   local v = LrView.osFactory()

   local note

   if colorspace_check.note then
      note = v:static_text { title = colorspace_check.note }
   else
      note = v:view {}
   end

   LrDialogs.presentModalDialog {
      title = LOC("$$$/146=Colorspace Warning"),
      resizable = true, -- why limit the user?
      save_frame = 'colorspace_warning_dialog_location',
      cancelVerb = "< exclude >", -- this magic, unpublished setting removes the cancel button, and may not work in the future
      contents = v:row {
         spacing = 20,

         v:picture {
            value = _PLUGIN:resourceId('colorspace_check.png'),
         },

         v:view {
            spacing = 13,

            v:static_text {
               title = LOC("$$$/147=Your currently-selected export settings will result in non-sRGB JPEGs to be uploaded.^n" ..
                           "Unless you have a specific reason to use a different color space, sRGB is recommended^n" ..
                           "for images intended for web-based viewing.")
            },

            note,

            v:row {
               v:static_text { title = LOC("$$$/148=For more information on digital-image color spaces:") },
               v:push_button {
                  title = LOC("$$$/149=visit here"),
                  action = function(button)
                              LrHttp.openUrlInBrowser("http://regex.info/blog/photo-tech/color-spaces-page1/")
                           end,
                  tooltip = LOC("$$$/110=Press to open ^1 in your browser", "http://regex.info/blog/photo-tech/color-spaces-page1/"),
               },
            },

            v:row {
               v:static_text { title = LOC("$$$/150=Show this warning when appropriate") },
               v:checkbox {
                  enabled = true,
                  value = LrView.bind { key = 'allow_colorspace_warning', object = state },
               },
            },
         },
      }
   }
end


function colorspace_check.synopsis(state)

   if state.allow_colorspace_warning and state._jpeg_colorspace_is_not_srgb then
      return LOC("$$$/151=COLOR SPACE WARNING")
   else
      return nil
   end

end


local function watch(state)

   if not state._update_status then
      -- must be being called from the "choose at export" dialog, so we'll just skip all this
      return
   end

   local function observe_space()
      state._jpeg_colorspace_is_not_srgb = (state.LR_format == 'JPEG' and state.LR_export_colorSpace ~= 'sRGB')
      state._update_status()
   end

   state:addObserver('LR_export_colorSpace', observe_space)
   state:addObserver('allow_colorspace_warning', observe_space)
   observe_space()


   local function observe_preference()
      state._disallow_colorspace_warning = not state.allow_colorspace_warning
   end

   state:addObserver('allow_colorspace_warning', observe_preference)

   observe_preference()

end

function colorspace_check.dialog_section(v, state)

   watch(state)

   return v:view {
      place = 'overlapping',

      v:view {
         place = 'horizontal',
         visible = LrBinding.andAllKeys('allow_colorspace_warning', '_jpeg_colorspace_is_not_srgb'),
         spacing = 0,
         margin_top = 10,
         margin_bottom = util.WinMac(0,2),

         v:static_text { title = LOC("$$$/152=Warning: do you really want to upload non-sRGB JPEGs?"),
                         text_color = LrColor(1,0,0),
                         font = "<system/bold>" },

         v:push_button { title = LOC("$$$/153=more info"),
                         color = LrColor(1,0,0),
                         font = "<system/small>",
                         action = function(button)
                                     colorspace_check.show_warning_dialog(state)
                                  end},
      },

      v:view {
         place = 'horizontal',
         visible = LrBinding.andAllKeys('_disallow_colorspace_warning', '_jpeg_colorspace_is_not_srgb'),
         spacing = 0,
         margin_top = 13,

         v:static_text {
            title = LOC("$$$/154=JPEG colorspace is not sRGB;  click to enable warnings: "),
            text_color = LrColor(.7,.7,.7),
         },
         v:checkbox {
            value = LrView.bind('allow_colorspace_warning')
         },
      },
   }
end


function colorspace_check.exportPresetFields()
   return {
      { key = 'allow_colorspace_warning', default = true }
   }
end


return function (note)
          colorspace_check.note = note
          return colorspace_check
       end
