--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--

-- Lightroom Export-API imports
local LrFunctionContext = import 'LrFunctionContext'   
local LrProgressScope   = import 'LrProgressScope'
local LrApplication     = import 'LrApplication'
local LrDialogs         = import 'LrDialogs'
local LrBinding         = import 'LrBinding'
local LrColor           = import 'LrColor'
local LrPrefs           = import 'LrPrefs'
local LrTasks           = import 'LrTasks'

local db_util = {}

local my = {}

function db_util.count_selected()

   local catalog = LrApplication.activeCatalog()
   local photos
   catalog:withCatalogDo(function()
                            photos = catalog.targetPhotos
                         end)

   local count = {
      old = 0,
      new = 0,
      all = 0,
   }

   if not photos or #photos == 0 then
      return count
   end


   local function check_one_photo(photo)
      local function _check_plugin_data(photoState)
         if photoState and photoState[my.ID] and #photoState[my.ID] > 0 then
            count.old = count.old + 1
         else
            count.new = count.new + 1
         end
         count.all = count.old + count.new
      end
      photo:withSettingsForPluginDo(_PLUGIN.id, _check_plugin_data)
   end

   for i, photo in ipairs(photos) do
      catalog:withCatalogDo(function()
                               check_one_photo(photo)
                            end)
      -- this seems prudent
      if i % 100 == 0 then
         LrTasks.yield()
      end
   end

   count.all = count.old + count.new

   return count
end

--
-- Given a function, will call it for each photo.
-- The function is called with two args, the photo object and the photo's plugin-speicifc settings table.
--
function db_util.with_each_photo(func)

   local catalog = LrApplication.activeCatalog()

   for i, photo in ipairs(catalog.targetPhotos) do
      catalog:withCatalogDo(function()
                               photo:withSettingsForPluginDo(_PLUGIN.id, function(settings)
                                                                            func(photo, settings)
                                                                         end)
                            end)
      -- this seems prudent
      if i % 100 == 0 then
         LrTasks.yield()
      end
   end
end


return function(args)
          my = args
          return db_util
       end