--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--
--
-- This is ugly -- I pounded this out in one short evening
--

local LrFunctionContext = import 'LrFunctionContext'
local LrFileUtils       = import 'LrFileUtils'
local LrBinding         = import 'LrBinding'
local LrDialogs         = import 'LrDialogs'
local LrColor           = import 'LrColor'
local LrPrefs           = import 'LrPrefs'
local LrShell           = import 'LrShell'
local LrView            = import 'LrView'
local util              = require 'util'

local my
local save_to_disk = {}

local HIGHLIGHT = LrColor(.6,.1,.3)
local RED       = LrColor(1,0,0)
local GRAY      = LrColor(.5,.5,.5)

local NotYetChosen = " < " .. LOC("$$$/111=not yet chosen") .. " > "

local function destination_folder(state)
   if state.save_after__dest_folder == nil then
      return NotYetChosen
   end

   if not state.save_after__put_in_subfolder or state.save_after__dest_subfolder == '' then
      return state.save_after__dest_folder 
   end

   return state.save_after__dest_folder .. util.WinMac("\\", "/") .. state.save_after__dest_subfolder
end


local function watch(state)

   local function update_status()
      if state.save_after__dest_folder then
         state.__save_after_dest_folder_name = state.save_after__dest_folder
         state.__save_after_dest_folder_color = HIGHLIGHT
      else
         state.__save_after_dest_folder_name = NotYetChosen
         state.__save_after_dest_folder_color = RED
      end

      state.__save_after_can_export = not state.save_after_upload or state.save_after__dest_folder ~= nil

      if not state.save_after_upload then
         state._save_after__synopsis = LOC("$$$/112=no")
      else
         state._save_after__synopsis = destination_folder(state)
      end
   end

   state:addObserver('save_after_upload',            update_status)
   state:addObserver('save_after__dest_folder',      update_status)
   state:addObserver('save_after__put_in_subfolder', update_status)
   state:addObserver('save_after__dest_subfolder',   update_status)
   update_status()

end


function DIALOG_GUTS(v,state)

   local prog = util.WinMac(LOC("$$$/Prog/Explorer=Explorer"),
                            LOC("$$$/Prog/Finder=Finder"))

   return v:view {
      visible = LrView.bind 'save_after_upload',
      bind_to_object = state,

      v:view {
         margin_bottom = util.WinMac(0,2),
         place = 'horizontal',

         v:static_text { title = LOC("$$$/113=Folder:") },
         v:static_text { title = LrView.bind "__save_after_dest_folder_name",
                         width_in_chars = tonumber(util.WinMac(LOC("$$$/113/width/Win=54"),
                                                               LOC("$$$/113/width/Mac=45"))),
                         truncation = 'middle', -- I'd like this to be 'head', but that doesn't seem to work
                         text_color = LrView.bind '__save_after_dest_folder_color'
                      },
         v:spacer { width = 4 },
         v:push_button { title = LOC("$$$/114=Choose^."),
                         action = function(button)
                                     local result = LrDialogs.runOpenPanel {
                                        title = LOC("$$$/115=Browse for Folder"),
                                        prompt = LOC("$$$/116=Select"),
                                        canChooseFiles = false,
                                        canChooseDirectories = true,
                                        canCreateDirectories = true,
                                        allowsMultipleSelection = false,
                                     }
                                     if result then
                                        state.save_after__dest_folder = result[1]
                                     end
                                  end,
                      }
      },

      v:spacer { height = 5 },

      v:view {
         visible = LrView.bind 'save_after_upload',      
         place = 'horizontal',
         v:checkbox { title = LOC("$$$/117=Put in Subfolder: "),
                      enabled = LrBinding.keyIsNotNil 'save_after__dest_folder',
                      value = LrView.bind 'save_after__put_in_subfolder' },

         v:view {
            place = 'vertical',
            v:edit_field { enabled = LrView.bind 'save_after__put_in_subfolder',
                           width_in_chars = tonumber(util.WinMac(LOC("$$$/117/width/Win=40"),LOC("$$$/117/width/Mac=40"))),
                           value = LrView.bind 'save_after__dest_subfolder',
                           immediate = true,
                           validate = function(view, value)
                                         value = string.gsub(value, '^%s+', '') -- remove leading whitespace
                                         value = string.gsub(value, '%s*$', '') -- remove trailing whitespace
                                         value = string.gsub(value, '[\\\/\:\*\?\"\<\>\|]+', '') -- remove items not allowed in a folder name

                                         return true, value, nil -- sigh, the changed value is not presented to the user until when it would have been done with immediate=false
                                      end},
         }
      },

      v:spacer { height = 5 },

      v:group_box {
         title = LOC("$$$/118=Options"),
         visible = LrView.bind 'save_after_upload',
         place = 'vertical',
         fill_horizontal = 1,

         v:view {
            place = 'horizontal',

            v:static_text { title = LOC("$$$/119=Show destination folder in ^1:", prog) },
            v:radio_button { title = LOC("$$$/112=no") .. "   ",
                             value = LrView.bind 'save_after__show_folder',
                             checked_value = false },
            v:radio_button { title = LOC("$$$/120=after first image") .. "   ",
                             value = LrView.bind 'save_after__show_folder',
                             checked_value = 'start' },
            v:radio_button { title = LOC("$$$/121=after last image"),
                             value = LrView.bind 'save_after__show_folder',
                             checked_value = 'end' },
         },


         v:view {
            place = 'horizontal',

            v:static_text { title = LOC("$$$/122=When files already exist at destination:") },

            v:radio_button { title = LOC("$$$/123=overwrite") .. "    ",
                             value = LrView.bind 'save_after__overwrite',
                             checked_value = true },
            v:radio_button { title = LOC("$$$/124=rename uniquely"),
                             value = LrView.bind 'save_after__overwrite',
                             checked_value = false },
         }
      }
   }

end


function save_to_disk.dialog_section(v, state)

   watch(state)

   return {
      title    = my.provider .. ": " .. LOC("$$$/125=Save to Local Disk"),
      synopsis = LrView.bind { key = '_save_after__synopsis', object = state },
      place = 'vertical',

      v:view {
         place = 'horizontal',
         margin = 3,
         fill_horizontal = 1,

         v:checkbox {
            title = "",
            value = LrView.bind 'save_after_upload',
         },

         v:static_text {
            title = " " .. LOC("$$$/126=After upload, also save images to local disk"),
         },

         v:spacer { width = 1, fill_horizontal = 1 },

         v:view {
            visible = LrView.bind 'save_after_upload',
            place = 'vertical',            
            fill_horizontal = 1,

            v:view {
               place = 'horizontal',

               v:static_text {
                  title = LOC("$$$/127=re-ask at the start of each export"),
               },
               v:checkbox {
                  title = "",
                  value = LrView.bind 'save_after__query_at_export_time',
               }
            },
            v:static_text { title = LOC("$$$/128=(useful for export presets)         "),
                            fill_horizontal = 1,
                            alignment = 'right',
                            text_color = GRAY }
         }
      },

      DIALOG_GUTS(v,state),
   }
end

local function export_time_query(context, state)

   local v = LrView.osFactory()

   local props = LrBinding.makePropertyTable(context)

   props.save_after__dest_folder      = state.save_after__dest_folder
   props.save_after__put_in_subfolder = state.save_after__put_in_subfolder
   props.save_after__dest_subfolder   = state.save_after__dest_subfolder
   props.save_after__overwrite        = state.save_after__overwrite
   props.save_after__show_folder      = state.save_after__show_folder

   props.save_after_upload = true -- this "save_after_upload" is local to this table, and is not copied to/from the main state table's "save_after_upload"

   watch(props)

   local UserReply = LrDialogs.presentModalDialog {
      title = LOC("$$$/129=Save Copy to Local Disk"),
      actionVerb = LOC("$$$/130=Continue With Export"),
      cancelVerb = LOC("$$$/131=Cancel Export"),
      bind_to_object = props,  -- huh?
      save_frame = 'save_to_disk__interactive_dialog_settings',
      resizable = true,
      contents = v:view {
         bind_to_object = props,
         v:view {
            place = 'horizontal',
            v:checkbox {
               title = "",
               value = LrView.bind 'save_after_upload',
            },
            v:view {
               place = 'overlapping',
               v:static_text { title = LOC("$$$/132=Save images to local disk during this export"),
                               visible = LrBinding.keyIsNotNil 'save_after__dest_folder' },
               v:static_text { title = LOC("$$$/133=images won't be saved because folder is not yet chosen"),
                               visible = LrBinding.keyIsNil 'save_after__dest_folder',
                               text_color = RED },
            }
         },

         v:spacer { height = 10 },
         
         DIALOG_GUTS(v,props)
      }
   }

   if UserReply ~= 'ok' then
      return nil
   end

   if not props.save_after_upload then
      state._save_after_upload_this_time = false
      return true
   end

   if not props.save_after__dest_folder then 
      return nil
   end
      
   state.save_after__dest_folder      = props.save_after__dest_folder
   state.save_after__put_in_subfolder = props.save_after__put_in_subfolder
   state.save_after__dest_subfolder   = props.save_after__dest_subfolder
   state.save_after__overwrite        = props.save_after__overwrite
   state.save_after__show_folder      = props.save_after__show_folder

   -- Changes we make in processRenderedPhotos to exportContext.propertyTable (AKA "state")
   -- do not necessarily get reflected back to the LR catalog, so we have to be sure to
   -- save them via LrPrefs as well
   local registeredState = LrPrefs.prefsForPlugin(_PLUGIN.id)

   registeredState.save_after__dest_folder      = props.save_after__dest_folder
   registeredState.save_after__put_in_subfolder = props.save_after__put_in_subfolder
   registeredState.save_after__dest_subfolder   = props.save_after__dest_subfolder
   registeredState.save_after__overwrite        = props.save_after__overwrite
   registeredState.save_after__show_folder      = props.save_after__show_folder

   return true
end



function save_to_disk.preRenderStuff(state)

   state._save_after_upload_this_time = state.save_after_upload
   state.__save_after__save_to_folder = nil -- clear it out just in case; we'll set it later
   state.__save_after__show_folder_at_next_file = nil
   state.__save_after__first_file_path = nil

   if not state.save_after_upload then
      return true -- no error
   end

   if state.save_after__query_at_export_time then
      local okay = false
      LrFunctionContext.callWithContext("custom dialog", function(context)
                                                            okay = export_time_query(context, state)
                                                         end)

      if not okay then
         return false -- error
      end
   end

   if not state._save_after_upload_this_time then
      return true -- no error
   end


   local dest = destination_folder(state)

   if not LrFileUtils.exists(dest) then
      LrFileUtils.createAllDirectories(dest)
   end

   state.__save_after__save_to_folder = dest

   if state.save_after__show_folder and state.save_after__show_folder == 'start' then
      state.__save_after__show_folder_at_next_file = true
   end

   return true -- no error
end

function save_to_disk.postRenderStuff(state)

   if not state._save_after_upload_this_time or not state.__save_after__save_to_folder then
      return
   end

   if state.save_after__show_folder and state.save_after__show_folder == 'end' and state.__save_after__first_file_path then
      LrShell.revealInShell(state.__save_after__first_file_path)
   end

end


function save_to_disk.handleFile(state, full_source_path, simple_name)

   if not state._save_after_upload_this_time or not state.__save_after__save_to_folder then
      --
      -- Delete the temporary file LR created for the upload.
      -- Lightroom will clear these out later for us, but getting rid of it
      -- avoids running out of space in the temporary folder
      --
      LrFileUtils.delete(full_source_path)
      return
   end

   local full_dest_path = state.__save_after__save_to_folder .. util.WinMac("\\", "/") .. simple_name

   if state.save_after__overwrite then
      -- I'm not sure what happens with the move/copy when the destination
      -- exists, so I'll try to delete it, just in case it's there
      LrFileUtils.delete(full_dest_path)
   else
      full_dest_path = LrFileUtils.chooseUniqueFileName(full_dest_path)
   end

   if LrFileUtils.pathsAreOnSameVolume(full_source_path, full_dest_path) then
      LrFileUtils.move(full_source_path, full_dest_path)
   else
      LrFileUtils.copy(full_source_path, full_dest_path)
   end

   if state.__save_after__show_folder_at_next_file then
      state.__save_after__show_folder_at_next_file = nil
      LrShell.revealInShell(full_dest_path)
   end

   if not state.__save_after__first_file_path then
      state.__save_after__first_file_path = full_dest_path
   end

end

function save_to_disk.exportPresetFields()

   return {
      { key = 'save_after_upload',                default = false    },
      { key = 'save_after__dest_folder',          default = nil      },
      { key = 'save_after__put_in_subfolder',     default = false    },
      { key = 'save_after__dest_subfolder',       default = ''       },
      { key = 'save_after__overwrite',            default = true     },
      { key = 'save_after__show_folder',          default = false    },
      { key = 'save_after__query_at_export_time', default = false    }
   }

end

return function (opts)
          my = opts
          return save_to_disk
       end
