--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--

local LrDialogs  = import 'LrDialogs'
local LrView     = import 'LrView'
local db_util -- filled in later

local upload_filter = {}
local my = {}

function upload_filter.init(state)
   state._count = db_util.count_selected()
end

function upload_filter.exportPresetFields()

   return {
      { key = 'upload_filter',   default = 'all' }
   }

end

function upload_filter.dialog_section(v, state)

   local heading
   if state._count.all == 1 then
      heading = LOC("$$$/393=One image has been selected for export^.")
   else
      heading = LOC("$$$/394=Of the ^1 images selected for export, process^.", state._count.all)
   end

   function watch()
      if state.upload_filter == 'all' then

         state._upload_filter_synopsis = nil
         state._upload_filter_can_do_old = true

      elseif state.upload_filter == 'old' then

         state._upload_filter_can_do_old = true
         if state._count.old == 0 then
            state._upload_filter_synopsis = LOC("$$$/395=refresh only (NONE)")
         else
            state._upload_filter_synopsis = LOC("$$$/396=refresh only (^1)", tostring(state._count.old))
         end

      else

         state._upload_filter_can_do_old = false
         if state._count.new then
            state._upload_filter_synopsis = LOC("$$$/397=new only (NONE)")
         else
            state._upload_filter_synopsis = LOC("$$$/398=new only (^1)", tostring(state._count.new))
         end

      end
   end

   state:addObserver('upload_filter', watch)
   watch()

   return v:view {
      spacing = 2,
      v:static_text { title = heading },
      v:view {
         margin_left = 30,
         v:view {
            place = 'horizontal',
            spacing = 0,

            v:radio_button {
               title   = "",
               value   = LrView.bind('upload_filter'),
               checked_value = 'all',
            },
            v:static_text { title = uLOC("$$$/399=All"), font = "<system/bold>" }, 
            v:static_text { title = ":" },
            v:static_text { title = LOC("$$$/400=process all selected images ( ^1 )", state._count.all) },
         },
         v:view {
            place = 'horizontal',
            spacing = 0,
            v:radio_button {
               title   = "",
               value   = LrView.bind('upload_filter'),
               checked_value = 'old',
            },
            v:static_text { title = LOC("$$$/401=Refresh"), font = "<system/bold>" }, 
            v:static_text { title = ":" },
            v:static_text { title = LOC("$$$/402=process only selected images previously exported to ^1 ( ^2 )", my.provider, state._count.old) },
         },
         v:view {
            place = 'horizontal',
            spacing = 0,
            v:radio_button {
               title   = "",
               value   = LrView.bind('upload_filter'),
               checked_value = 'new',
            },
            v:static_text { title = uLOC("$$$/403=New"), font = "<system/bold>" }, 
            v:static_text { title = ":" },
            v:static_text { title = LOC("$$$/404=process only selected images not previously exported to ^1 ( ^2 )", my.provider, state._count.new) },
         },
      },
   }

end

--
-- Returns true if there are still images left to export, false if all have been filtered out.
--
function upload_filter.apply(exportContext)

   local state = exportContext.propertyTable

   if state.upload_filter == 'all' then
      return true
   end

   db_util.with_each_photo(function(photo, photoState)
                              local thisPhoto
                              if photoState and photoState[my.ID] and #photoState[my.ID] > 0 then
                                 thisPhoto = 'old'
                              else
                                 thisPhoto = 'new'
                              end
                              if thisPhoto ~= state.upload_filter then
                                 exportContext.exportSession:removePhoto(photo)
                              end
                           end)

   if exportContext.exportSession:countRenditions() == 0 then

      LrDialogs.message(LOC("$$$/405=No images to process"),
                        LOC("$$$/406=All were filtered out"))
      return false

   else

      return true

   end
end

return function(args)
          my = args
          db_util = (require 'db-util')(args)
          return upload_filter
       end